#!/bin/sh
# This script gets X key mapping data of the current system and append it to 
# a pwkrc file. Default pwkrc files is /etc/pwkrc file. 
# You can use option -c to change its output to other filename.
# It looks up into xmodmap to finds out xf86 keycodes for each Palm Wirelss 
# Keyboard key. It needs irdatagen.
#
# Copyright GPL. 2004.12.25. lampo1@paran.com

# xmodmap -pke :
# 
#       field2  |  field4  |     field5
#       ==================================
#       keycode |  xname   |  shifted xname
# 
#


# pwkrc file :
#
#  field3  | field4      | field5      | field10 | field11     | field12 
#  =======================================================================
#   xname  | LFNed xname | RFNed xname | kcode  | LFN kcodes | RFN kcodes 
#		
#
#  - If xname is found in the field4 of xmodmap, it means the key's 
#    normal use(= no FN combination) should generate keycode of a 
#    standard(= not shifted) key.
#
#  - If LFNed xname is found in the field4 of xmodmap, it means the key's 
#    LFN combination should generate keycode of a standard(= not shifted) 
#    key. 
#
#  - If LFNed xname is found in the field5 of xmodmap, it means the key's 
#    LFN combination should generate keycode of a shifted key. So, it
#    needs shift emulation.
#
#  - If RFNed xname is found in the field4 of xmodmap, it means the key's 
#    RFN combination should generate keycode of a standard(= not shifted) 
#    key. 
#
#  - If RFNed xname is found in the field5 of xmodmap, it means the key's 
#    LFN combination should generate keycode of a shifted key. So, it
#    needs shift emulation.

if [ $# = "0" ]
then
    echo "help : keycodegen.sh -h"
    exit 1
fi

while getopts ":f:o:ch" Option
do
    case $Option in
	f ) if [ -r $OPTARG ] && [ -f $OPTARG ]
	    then
		infile=$OPTARG
	    else
		echo "No such input file $OPTARG"
		exit 1 
	    fi ;;
	o ) outfile=$OPTARG ;;
	h ) echo "This script add current Xwindow's keycode mapping "
	    echo "to an input pwkrc file and then generate pwkrc file "
	    echo "for pwkxwin driver."
	    echo " "
	    echo "usage: keycodegen [-f inputfile] [-o outputfile]"
	    echo 
 	    exit 1
	    ;;
	* ) exit 1 ;;
    esac
done

# First, Check whether all standard keys are supported on this X window system
# and find out shift keycode mapping.

set +u
for xname in `./irdatagen -s | awk ' ( /^[0-9]/) && ($1 >= 5) {print $3}'`
do
    echo $xname
    if xmodmap -pke | awk ' {print $2 " " $4 " "}' | grep -c " "$xname" "
    then 
        echo Good.
    else
        echo Some standard key mappings are not defined on your Xwindow system.
        exit 1
    fi 
done

echo "Copying Xwindow key mappings into a pwkrc file..."

shiftcode=` xmodmap -pke | awk ' $4 == "Shift_L" { print $2 }' `
echo "Shift keycode = $shiftcode"

if [ -e temp?.tmp ] ; then
    rm temp?.tmp
fi

cp $infile ./baredata.tmp
xmodmap -pke | awk ' NF >= 4 {print $2, $4, $5}' > xmodmap.tmp

while read field2 field4 field5; do
echo $field2 $field4 $field5
awk ' $3 != "none" && $3 == f4 { print $1,$2,$3,"field10",f2 ":0:0:" >> "temp1.tmp"}
      $4 != "none" && $4 == f4 { print $1,$2,$4,"field11",f2 ":0:0:" >> "temp2.tmp"}
      $4 != "none" && $4 == f5 { if ($1%2 == 1) 
				     print $1,$2,$4,"field11",shift ":" f2 ":0:" \
				         >> "temp2.tmp" 
			         else 
				     print $1,$2,$4,"field11",f2 ":" shift ":0:" \
				         >> "temp2.tmp"  
	                       }			    
      $5 != "none" && $5 == f4 { print $1,$2,$5,"field12",f2 ":0:0:" >> "temp3.tmp"}
      $5 != "none" && $5 == f5 { if ($1%2 == 1) 
			             print $1,$2,$5,"field12",shift ":" f2 ":0:" \
				         >> "temp3.tmp"
			         else 
			             print $1,$2,$5,"field12",f2 ":" shift ":0:" \
				         >> "temp3.tmp" 
	                       }' f2=$field2 f4=$field4 f5=$field5 \
			          shift=$shiftcode baredata.tmp
done  < ./xmodmap.tmp

echo "processing..."
awk ' BEGIN { while (getline x < "temp1.tmp" > 0) field10[++n] = x 
	      max_idx10 = n

	      n = 0
	      while (getline x < "temp2.tmp" > 0) field11[++n] = x  
	      max_idx11 = n

	      n = 0
	      while (getline x < "temp3.tmp" > 0) field12[++n] = x
	      max_idx12 = n
	    }

      $1 !~  /^[0-9]+/ { print $0 }
      $1 ~ /^[0-9]+/   { for ( i = 1; i <= max_idx10; i++) { 
	                     split(field10[i], arr)
		             if ( $1 == arr["1"] ) {
			         $10 = arr["5"]
				 break
                             }
			 }

			 for ( i = 1; i <= max_idx11; i++) { 
	                     split(field11[i], arr)
		             if ( $1 == arr["1"] ) {
			         $11 = arr["5"]
				 break
                             }
			 }

			 for ( i = 1; i <= max_idx12; i++) { 
	                     split(field12[i], arr)
		             if ( $1 == arr["1"] ) {
			         $12 = arr["5"]
				 break
                             }
			 }
	                 print $0
                       }' $infile  | sort +0 -g -o data.tmp

if [ -f $outfile ] ; then 
    echo "File $outfile already exists. It will be renamed as $outfile~"
    cp ./$outfile $outfile~
fi

cp ./data.tmp $outfile
rm *.tmp

echo "New $outfile is generated." 
echo "Done."












